scrPath = File::dirname(__FILE__.gsub(/\\/, '/')) + '/'
require scrPath + 'datafile.rb'
#require 'pp'

module OED

class DDDFile < File

  attr_reader :startID, :idCount

  def init(root)
    @root = root.sub(/[\/\\]$/, '') + '/'

    seek(44)

    @table = []
    while true do
      type, = read(4).unpack('Z4')
#p type
      break if type.empty?
      size, pos, = read(16).unpack('V4')
      case type
      when '_dv'
        curpos = tell()
        seek(pos)
        read(32)
        @startID, @idCount = read(8).unpack('VV')
        seek(curpos)
      when '_dk'
        @docSegpos = pos
      when '_dl'
        @fnameSegpos = pos
      when 'hTm'
        @entrySegpos = pos
      end
      @table.push([type, size, pos])
    end
  end

  def getinfo(id)
    return nil if (id < @startID) or (id >= @startID + @idCount)

    seek(@docSegpos + (id - @startID) * 14)
    pos, size, fnofs, fnsize = read(14).unpack('VVVv')
    seek(@fnameSegpos + fnofs)
    fname, = read(fnsize).unpack('Z*')
    seek(@entrySegpos + (id - @startID) * 7)
    entryID, = (read(7) + "\x00").unpack('VV')

    return pos, size, fname, entryID
  end

  def get(id)
    pos, size, fname, = getinfo(id)
    return nil if pos.nil?

    num = File::basename(fname, '.htm').to_i - 1
    dataf = createDataFileObject(@root + fname, num)
    return nil if dataf.nil?

    version = dataf.setup
    if version == V_UNKNOWN
      dataf.seek(pos + dataf.offset)
      data = dataf.read(size)
    else
      data = dataf.decode(pos, size)
    end
    dataf.close
    return data
  end

end

end ##### module ################################################

if __FILE__ == $0

  include OED

  root = 'H:/OED310/'
  partsDir = root + 'Collects/main/parts/'
  htmname = ''; doprint = false; prevpos = 0; count = 0
#  60.downto(1) do |n|    # v3.0x
  61.downto(1) do |n|    # v3.1

    infname = sprintf('%08d.ddd', n)
    inf = DDDFile.open(partsDir + infname, 'rb')
    inf.init(root)
    (inf.startID + inf.idCount - 1).downto(inf.startID) do |id|
      pos, size, fname, entryID = inf.getinfo(id)
      if fname != htmname
        puts '%s --------' % [fname]
        htmname = fname
        doprint = true
        prevpos = 0
        count = 0
      end
      if doprint
        if pos >= 127 * 123
          doprint = false
#break
        else
          count += 1
          dif = pos - prevpos
          puts '%02d: %07d at %5d %5d %+d' % [count, id, pos, size, dif]
#          puts inf.get(id)
          prevpos = pos + size
        end
      end
    end

    inf.close
#break
  end

end

__END__
infname = ARGV.shift
inf = open(infname, 'rb')

data = inf.read(4)
data.each_byte do |b|
  printf('%02X ', b)
end
puts ''

data = inf.read(40)
data.each_byte do |b|
  printf('%02X ', b)
end
puts ''

a = []; ofs = []
tbl = []
while true do
  type, = inf.read(4).unpack('Z4')
  break if type.empty?
  a = inf.read(16).unpack('V4')
#  ofs.push(a[1])
  tbl.push([type, a[0], a[1]])
  printf("%-4s %08X(%7d) %08X(%7d) %08X %08X\n", type, a[0], a[0], a[1], a[1], a[2], a[3])
#  printf("     %8d %8d %8d %8d\n", a[0], a[1], a[2], a[3])
end

outfname = File::basename(infname)

startID = count = 0
tbl.each do |type, size, pos|
  case type
  when '_dv'
    inf.seek(pos)
    inf.read(32)
    startID, count = inf.read(8).unpack('VV')
  end
end

#p startID, count
#p tbl

tbl.each do |type, size, pos|
  case type
  when '_dk'
    outf = open(outfname + type, 'w')
    inf.seek(pos)
    count.times do |n|
      break if n > 100
      a = inf.read(14).unpack('VVVv')
      outf.printf("%5d: %08X(%8d) %08X(%8d) %08X(%8d) %04X(%5d)\n", n, a[0], a[0], a[1], a[1], a[2], a[2], a[3], a[3])
    end
    outf.close
  when 'aaa', 'hTk', 'hTl', 'hTn', 'hTm'
    outf = open(outfname + type, 'w')
    inf.seek(pos)
    count.times do |n|
      break if n > 100
      a = (inf.read(7) + "\x00").unpack('VV')
      outf.printf("%5d: %08X(%8d) %06X(%8d)\n", n, a[0], a[0], a[1], a[1])
    end
    outf.close
  when 'hTo', 'hTp'
    outf = open(outfname + type, 'w')
    inf.seek(pos)
    count.times do |n|
      break if n > 100
      a = (inf.read(3) + "\x00").unpack('V')
      b = inf.read(6).unpack('Vv')
      outf.printf("%5d: %06X(%8d) %08X(%8d) %04X(%8d)\n", n, a[0], a[0], b[0], b[0], b[1], b[1])
    end
    outf.close
  end
end

=begin
ofs.size.times do |n|
  if ofs[n + 1] == 0
    param = sprintf('/s %X %s > %s_%02d', ofs[n], infname, outfname, n)
  else
    param = sprintf('/s %X /e %X %s > %s_%02d', ofs[n], ofs[n + 1], infname, outfname, n)
  end
  puts param
  system('dump.cmd', param)
end
=end

inf.close
